# -*- coding: utf-8 -*-
"""Check-JVM

Check-JVM is a Python package that provide tools and
functions for checking Oracle JAVA installations on
Windows and Linux servers.
"""

# MIT License (see LICENSE)
# Author: Dániel Hagyárossy <d.hagyarossy@sapstar.eu>
